@@ -3,9 +3,9 @@
 --  Author : Manuel M T Chakravarty
 --  Created: 16 August 99
 --
---  Version $Revision: 1.24 $ from $Date: 2003/10/19 10:46:10 $
+--  Version $Revision: 1.26 $ from $Date: 2004/10/17 08:31:08 $
 --
---  Copyright (c) [1999..2003] Manuel M T Chakravarty
+--  Copyright (c) [1999..2004] Manuel M T Chakravarty
 --
 --  This file is free software; you can redistribute it and/or modify
 --  it under the terms of the GNU General Public License as published by
@@ -374,7 +374,7 @@ dumpCHS fname mod pureHaskell  =
   where
     contents version kind = 
       "-- GENERATED by " ++ version ++ " " ++ kind ++ "\n\
-      \-- ** Edit the orignal .chs file instead!\n\n"
+      \-- Edit the ORIGNAL .chs file instead!\n\n"
       ++ showCHSModule mod pureHaskell
 
 -- to keep track of the current state of the line emission automaton
@@ -408,8 +408,8 @@ showCHSModule (CHSModule frags) pureHaskell  =
 	nextState	 = if generated then Wait else NoLine
       in
 	(if emitNow then
-	   showString ("{-# LINE " ++ show (line `max` 0) ++ " " ++ show fname 
-		       ++ " #-}\n")
+	   showString ("\n{-# LINE " ++ show (line `max` 0) ++ " " ++ 
+		       show fname ++ " #-}")
 	 else id)
       . showString s
       . showFrags pureHs nextState frags
@@ -421,12 +421,12 @@ showCHSModule (CHSModule frags) pureHaskell  =
     showFrags False  _     (CHSCPP  s    _     : frags) =   
         showChar '#'
       . showString s
-      . showChar '\n'
+--      . showChar '\n'
       . showFrags False Emit frags
     showFrags False  _     (CHSC    s    _     : frags) =
-        showString "\n#c\n"
+        showString "\n#c"
       . showString s
-      . showString "\n#endc\n"
+      . showString "\n#endc"
       . showFrags False Emit frags
     showFrags False  _     (CHSCond _    _     : frags) =
       interr "showCHSFrag: Cannot print `CHSCond'!"
@@ -974,7 +974,7 @@ parseOptPrefix _     (CHSTokPrefix _:toks) = syntaxError toks
 parseOptPrefix _     toks		   = return (Nothing, toks)
 
 -- first argument is the identifier that is to be used when `^' is given and
--- the second indicates whether the first characters has to be upper case
+-- the second indicates whether the first character has to be upper case
 --
 parseOptAs :: Ident -> Bool -> [CHSToken] -> CST s (Maybe Ident, [CHSToken])
 parseOptAs _   _     (CHSTokAs _:CHSTokIdent _ ide:toks) = 
