@@ -3,9 +3,9 @@
 --  Author : Manuel M T Chakravarty
 --  Created: 13 August 99
 --
---  Version $Revision: 1.19 $ from $Date: 2003/04/16 11:13:10 $
+--  Version $Revision: 1.20 $ from $Date: 2004/10/17 08:31:08 $
 --
---  Copyright (c) [1999..2003] Manuel M T Chakravarty
+--  Copyright (c) [1999..2004] Manuel M T Chakravarty
 --
 --  This file is free software; you can redistribute it and/or modify
 --  it under the terms of the GNU General Public License as published by
@@ -88,7 +88,8 @@
 --  * On encountering the lexeme `{#', a meta action in the base lexer
 --    transfers control to the following binding-hook lexer:
 --
---      ident       -> letter (letter | digit | `'')*
+--      ident       -> letter (letter | digit | `\'')*
+--		     | `\'' letter (letter | digit)* `\''
 --      reservedid  -> `as' | `call' | `class' | `context' | `deriving' 
 --		     | `enum' | `foreign' | `fun' | `get' | `lib' 
 --		     | `newtype' | `pointer' | `prefix' | `pure' | `set'
@@ -103,6 +104,8 @@
 --
 --    Control characters, white space, and comments are discarded in the
 --    binding-hook lexer.  Nested comments are not allowed in a binding hook.
+--    Identifiers can be enclosed in single quotes to avoid collision with
+--    C->Haskell keywords.
 --
 --  * In the binding-hook lexer, the lexeme `#}' transfers control back to the 
 --    base lexer.  An occurence of the lexeme `{#' inside the binding-hook
@@ -124,7 +127,9 @@
 --    the tool, as illegal identifier will never occur in a C header file that
 --    is accepted by the C lexer.  In the case of Haskell identifiers, a
 --    confusion between variable and constructor identifiers will be noted by
---    the Haskell compiler translating the code generated by c2hs.
+--    the Haskell compiler translating the code generated by c2hs.  Moreover,
+--    identifiers can be enclosed in single quotes to avoid collision with
+--    C->Haskell keywords, but those may not contain apostrophes.
 --
 --  * Any line starting with the character `#' is regarded to be a C
 --    preprocessor directive.  With the exception of `#c' and `#endc', which
@@ -566,15 +571,18 @@ cpp :: CHSLexer
 cpp = directive
       where
         directive = 
-	  string "\n#" +> alt ('\t':inlineSet)`star` char '\n'
+	  string "\n#" +> alt ('\t':inlineSet)`star` epsilon
 	  `lexmeta` 
 	     \(_:_:dir) pos s ->	-- strip off the "\n#"
 	       case dir of
-	         'c':sp:_ | sp `elem` " \t\n" ->	-- #c
-		   (Nothing, retPos (retPos pos), s, Just cLexer)
+	         ['c']                      ->		-- #c
+		   (Nothing, retPos pos, s, Just cLexer)
+                 -- a #c may be followed by whitespace
+	         'c':sp:_ | sp `elem` " \t" ->		-- #c
+		   (Nothing, retPos pos, s, Just cLexer)
                  _                            ->        -- CPP directive
-		   (Just $ Right (CHSTokCPP pos (init dir)), 
-		    retPos (retPos pos), s, Nothing)
+		   (Just $ Right (CHSTokCPP pos dir), 
+		    retPos pos, s, Nothing)
 
 -- the binding hook lexer
 --
@@ -625,8 +633,13 @@ identOrKW :: CHSLexer
 -- the strictness annotations seem to help a bit
 --
 identOrKW  = 
-  letter +> (letter >|< digit >|< char '\'')`star` epsilon
-  `lexactionName` \cs pos name -> (idkwtok $!pos) cs name
+       -- identifier or keyword
+       (letter +> (letter >|< digit >|< char '\'')`star` epsilon
+       `lexactionName` \cs pos name -> (idkwtok $!pos) cs name)
+  >||< -- identifier in single quotes
+       (char '\'' +> letter +> (letter >|< digit)`star` char '\''
+       `lexactionName` \cs pos name -> (mkid $!pos) cs name)
+       -- NB: quotes are removed by lexemeToIdent
   where
     idkwtok pos "as"               _    = CHSTokAs      pos
     idkwtok pos "call"             _    = CHSTokCall    pos
@@ -651,8 +664,9 @@ identOrKW  =
     idkwtok pos "underscoreToCase" _    = CHSTok_2Case  pos
     idkwtok pos "unsafe"           _    = CHSTokUnsafe  pos
     idkwtok pos "with"             _    = CHSTokWith    pos
-    idkwtok pos cs                 name = CHSTokIdent   
-					    pos (lexemeToIdent pos cs name)
+    idkwtok pos cs                 name = mkid pos cs name
+    --
+    mkid pos cs name = CHSTokIdent pos (lexemeToIdent pos cs name)
 
 -- reserved symbols
 --
