@@ -3,7 +3,7 @@
 --  Author : Manuel M T Chakravarty
 --  Created: 17 August 99
 --
---  Version $Revision: 1.49 $ from $Date: 2003/05/22 04:15:02 $
+--  Version $Revision: 1.53 $ from $Date: 2004/10/17 08:31:08 $
 --
 --  Copyright (c) [1999..2003] Manuel M T Chakravarty
 --
@@ -153,7 +153,7 @@ import CHS	  (CHSModule(..), CHSFrag(..), CHSHook(..), CHSTrans(..),
 import CInfo      (CPrimType(..), size, alignment, bitfieldIntSigned,
 		   bitfieldAlignment)
 import GBMonad    (TransFun, transTabToTransFun, HsObject(..), GB,
-		   initialGBState, setContext, getLibrary, getPrefix, 
+		   initialGBState, setContext, getPrefix, 
 		   delayCode, getDelayedCode, ptrMapsTo, queryPtr, objIs,
 		   queryObj, queryClass, queryPointer, mergeMaps, dumpMaps)
 
@@ -163,7 +163,8 @@ import GBMonad    (TransFun, transTabToTransFun, HsObject(..), GB,
 
 -- FIXME: 
 -- - we might have a dynamically extended table in the monad if needed (we
---   could marshall enums this way)
+--   could marshall enums this way and also save the `id' marshallers for
+--   pointers defined via (newtype) pointer hooks)
 -- - the checks for the Haskell types are quite kludgy
 
 -- determine the default "in" marshaller for the given Haskell and C types
@@ -246,9 +247,14 @@ isFloatHsType _	       = False
 
 -- check for integral C types
 --
+-- * For marshalling purposes C char's are integral types (see also types
+--   classes for which the FFI guarantees instances for `CChar', `CSChar', and
+--   `CUChar')
+--
 isIntegralCPrimType :: CPrimType -> Bool
-isIntegralCPrimType  = (`elem` [CIntPT, CShortPT, CLongPT, CLLongPT, CUIntPT, 
-			        CUShortPT, CULongPT, CULLongPT])
+isIntegralCPrimType  = (`elem` [CCharPT, CSCharPT, CIntPT, CShortPT, CLongPT,
+				CLLongPT, CUIntPT, CUCharPT, CUShortPT,
+				CULongPT, CULLongPT]) 
 
 -- check for floating C types
 --
@@ -439,14 +445,14 @@ expandHook hook@(CHSFun isPure isUns ide oalias ctxt parms parm pos) =
     -- function; we use shadow identifiers, so the returned identifier is used 
     -- afterwards instead of the original one
     --
-    (ObjCO cdecl, ide) <- findFunObj ide True
+    (ObjCO cdecl, cide) <- findFunObj ide True
     let ideLexeme = identToLexeme ide  -- orignal name might have been a shadow
 	hsLexeme  = ideLexeme `maybe` identToLexeme $ oalias
 	fiLexeme  = hsLexeme ++ "'_"   -- *Urgh* - probably unqiue...
 	fiIde     = onlyPosIdent nopos fiLexeme
-        cdecl'    = ide `simplifyDecl` cdecl
-	callHook  = CHSCall isPure isUns ide (Just fiIde) pos
-    callImport callHook isPure isUns ideLexeme fiLexeme cdecl' pos
+        cdecl'    = cide `simplifyDecl` cdecl
+	callHook  = CHSCall isPure isUns cide (Just fiIde) pos
+    callImport callHook isPure isUns (identToLexeme cide) fiLexeme cdecl' pos
     funDef isPure hsLexeme fiLexeme cdecl' ctxt parms parm pos
   where
     traceEnter = traceGenBind $ 
@@ -657,12 +663,12 @@ callImport :: CHSHook -> Bool -> Bool -> String -> String -> CDecl -> Position
 	   -> GB ()
 callImport hook isPure isUns ideLexeme hsLexeme cdecl pos =
   do
-    -- compute the external type from the declaration, get the library, and
-    -- delay the foreign export declaration
+    -- compute the external type from the declaration, and delay the foreign
+    -- export declaration
     --
     extType <- extractFunType pos cdecl isPure
-    lib     <- getLibrary
-    delayCode hook (foreignImport lib ideLexeme hsLexeme isUns extType)
+    header  <- getSwitch headerSB
+    delayCode hook (foreignImport header ideLexeme hsLexeme isUns extType)
     traceFunType extType
   where
     traceFunType et = traceGenBind $ 
@@ -670,17 +676,12 @@ callImport hook isPure isUns ideLexeme hsLexeme cdecl pos =
 
 -- Haskell code for the foreign import declaration needed by a call hook
 --
--- * appends a configuration dependent library suffix `dlsuffix'
---
 foreignImport :: String -> String -> String -> Bool -> ExtType -> String
-foreignImport lib ident hsIdent isUnsafe ty  =
-  "foreign import ccall " ++ libName ++ "\"" ++ ident ++ "\"" ++ maybeUnsafe 
-  ++ "\n  " ++ hsIdent ++ " :: " ++ showExtType ty ++ "\n"
+foreignImport header ident hsIdent isUnsafe ty  =
+  "foreign import ccall " ++ safety ++ " \"" ++ header ++ " " ++ ident ++ 
+  "\"\n  " ++ hsIdent ++ " :: " ++ showExtType ty ++ "\n"
   where
---    libName	= if null lib then "" else "\"" ++ lib ++ dlsuffix ++ "\" "
-    libName = ""
--- FIXME: libName removed until the new FFI conventions for libs are impl.
-    maybeUnsafe = if isUnsafe then " unsafe" else ""
+    safety = if isUnsafe then "unsafe" else "safe"
 
 -- produce a Haskell function definition for a fun hook
 --
@@ -689,7 +690,7 @@ funDef :: Bool		     -- pure function?
        -> String	     -- Haskell name of the foreign imported C function
        -> CDecl		     -- simplified declaration of the C function
        -> Maybe String	     -- type context of the new Haskell function
-       -> [CHSParm]	     -- parameter marhsalling description
+       -> [CHSParm]	     -- parameter marshalling description
        -> CHSParm	     -- result marshalling description 
        -> Position	     -- source location of the hook
        -> GB String	     -- Haskell code in text form
@@ -861,7 +862,7 @@ addDftMarshaller pos parms parm cdecl = do
     addDftIn pos imMarsh@Nothing          hsTy cTys = do
       marsh <- lookupDftMarshIn hsTy cTys
       when (isNothing marsh) $
-        noDftMarshErr pos "\"in\""
+        noDftMarshErr pos "\"in\"" hsTy cTys
       return (marsh, case marsh of {Just (_, kind) -> kind == CHSIOArg})
     --
     addDftOut _   omMarsh@(Just (_, kind)) _    _    = return (omMarsh,
@@ -869,7 +870,7 @@ addDftMarshaller pos parms parm cdecl = do
     addDftOut pos omMarsh@Nothing          hsTy cTys = do
       marsh <- lookupDftMarshOut hsTy cTys
       when (isNothing marsh) $
-        noDftMarshErr pos "\"out\""
+        noDftMarshErr pos "\"out\"" hsTy cTys
       return (marsh, case marsh of {Just (_, kind) -> kind == CHSIOArg})
     --
     -- add void marshaller if no explict one is given
@@ -1086,12 +1087,23 @@ pointerDef isStar cNameFull hsName ptrKind isNewtype hsType isFun =
 	ptrType = ptrCon ++ " (" ++ ptrArg ++ ")"
 	thePtr  = (isStar, cNameFull)
     case ptrKind of
-      CHSForeignPtr -> thePtr `ptrMapsTo` (hsName, "Ptr (" ++ ptrArg ++ ")")
+      CHSForeignPtr -> thePtr `ptrMapsTo` ("Ptr (" ++ ptrArg ++ ")", 
+					   "Ptr (" ++ ptrArg ++ ")")
       _		    -> thePtr `ptrMapsTo` (hsName, hsName)
     return $
       if isNewtype 
-      then "newtype " ++ hsName ++ " = " ++ hsName ++ " (" ++ ptrType ++ ")"
+      then "newtype " ++ hsName ++ " = " ++ hsName ++ " (" ++ ptrType ++ ")" ++
+	   withForeignFun
       else "type "    ++ hsName ++ " = "                   ++ ptrType
+    where
+      -- if we have a foreign pointer wrapped into a newtype, provide a
+      -- safe unwrapping function automatically
+      --
+      withForeignFun 
+        | ptrKind == CHSForeignPtr = 
+	  "\n" ++
+          "with" ++ hsName ++ " (" ++ hsName ++ " fptr) = withForeignPtr fptr"
+	| otherwise		   = ""
 
 -- generate the class and instance definitions for a class hook
 --
@@ -1983,9 +1995,11 @@ marshArgMismatchErr pos reason  =
     ["Function arity mismatch!",
      reason]
 
-noDftMarshErr           :: Position -> String -> GB a
-noDftMarshErr pos inOut  =
+noDftMarshErr :: Position -> String -> String -> [ExtType] -> GB a
+noDftMarshErr pos inOut hsTy cTys  =
   raiseErrorCTExc pos
     ["Missing " ++ inOut ++ " marshaller!",
      "There is no default marshaller for this combination of Haskell and \
-     \C type available."]
+     \C type:",
+     "Haskell type: " ++ hsTy,
+     "C type      : " ++ concat (intersperse " " (map showExtType cTys))]
